<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly!
}

if ( ! class_exists( 'WPSC_Cron' ) ) :

	final class WPSC_Cron {

		/**
		 * Initialize this class
		 *
		 * @return void
		 */
		public static function init() {

			// Add custom cron intervals.
			add_filter( 'cron_schedules', array( __CLASS__, 'custom_interval' ) ); //phpcs:ignore

			// Schedule cron jobs.
			add_action( 'init', array( __CLASS__, 'schedule_events' ) );
		}

		/**
		 * Custom cron job intervals for SupportCandy
		 *
		 * @param array $schedules - schedule time.
		 * @return array
		 */
		public static function custom_interval( $schedules ) {

			$schedules['wpsc_1min'] = array(
				'interval' => 60,
				'display'  => esc_attr__( 'Every one minute', 'supportcandy' ),
			);

			$schedules['wpsc_5min'] = array(
				'interval' => 300,
				'display'  => esc_attr__( 'Every five minutes', 'supportcandy' ),
			);

			return $schedules;
		}

		/**
		 * Schedule cron job events for SupportCandy
		 *
		 * @return void
		 */
		public static function schedule_events() {

			// Schedule cron job for every minute events.
			if ( ! wp_next_scheduled( 'wpsc_cron_one_minute' ) ) {
				wp_schedule_event(
					time(),
					'wpsc_1min',
					'wpsc_cron_one_minute'
				);
			}

			// Schedule cron job for every five minute events.
			if ( ! wp_next_scheduled( 'wpsc_cron_five_minute' ) ) {
				wp_schedule_event(
					time(),
					'wpsc_5min',
					'wpsc_cron_five_minute'
				);
			}

			// Schedule cron job for daily events.
			if ( ! wp_next_scheduled( 'wpsc_cron_daily' ) ) {
				wp_schedule_event(
					self::get_midnight_timestamp(),
					'daily',
					'wpsc_cron_daily'
				);
			}
		}

		/**
		 * Remove existing scheduled events.
		 * Can be used while deactivation of plugin or resetting schedules after an update etc.
		 *
		 * @return void
		 */
		public static function unschedule_events() {

			// Remove every minute cron.
			$timestamp = wp_next_scheduled( 'wpsc_cron_one_minute' );
			if ( $timestamp ) {
				wp_unschedule_event( $timestamp, 'wpsc_cron_one_minute' );
			}

			// Remove every five minute cron.
			$timestamp = wp_next_scheduled( 'wpsc_cron_five_minute' );
			if ( $timestamp ) {
				wp_unschedule_event( $timestamp, 'wpsc_cron_five_minute' );
			}

			// Remove daily cron.
			$timestamp = wp_next_scheduled( 'wpsc_cron_daily' );
			if ( $timestamp ) {
				wp_unschedule_event( $timestamp, 'wpsc_cron_daily' );
			}
		}

		/**
		 * Provide mid-night unix timestamp
		 *
		 * @return String
		 */
		public static function get_midnight_timestamp() {

			$tz   = wp_timezone();
			$date = new DateTime( 'now', $tz );
			$date->setTime( 0, 0, 0 );
			$date->add( new DateInterval( 'P1D' ) );
			return $date->getTimestamp();
		}
	}
endif;

WPSC_Cron::init();
